// A02-Servos
/* Set servos to mid position, rollC and paceC to check if feet are flat and feet together      
     if not change rollC and paceC until they are
   Set servos maximum and minimum safe angles for Roll Right and Roll Left, and Pace Left Forward and Left Back    
   Use the Arduino Serial Monitor set to 'no line ending', type -, +, /, H, L, or C and press ENTER
       to send -, +, /, H, L, C  
   Or use a Serial Terminal at 9600 baud. 
     [I] to display instructions
     [*] to display current EEPROM values
     [+], [-] adjust Servo position
     [/] to change servo.
     [C] to set Centre angles RollC and PaceC
     [H],[L] to set High=Maximum angle and Low=Minimum angle
     [R] to set the default Roll amount 
     [P] to set the default Pace length
  instructions and current settings appear on screen.	   
*/	   
//Binary sketch size: 6,158 bytes (of a 14,336 byte maximum)/(of a 30,720 byte maximum)

#include <Servo.h>
/* Servo servoname - create servo object to control servo servoname  
   attach(pin) - Attaches a servo to an i/o pin.
   attach(pin,min,max) - Attaches to a pin setting min and max values in microseconds, default min is 544, max is 2400  
   write()     - Sets the servo angle in degrees.  (invalid angle that is valid as pulse in microseconds is treated as microseconds)
   writeMicroseconds() - Sets the servo pulse width in microseconds 
   read()      - Gets the last written servo pulse width as an angle between 0 and 180. 
   readMicroseconds()  - Gets the last written servo pulse width in microseconds. (was read_us() in first release)
   attached()  - Returns true if there is a servo attached. 
   detach()    - Stops an attached i/o pin from pulsing its servo. 
 */ 
#include <EEPROM.h>           // to read and write internal EEPROM
                              // EEPROM.write(address,bytevalue)
 		   	                  // bytevalue =EEPROM.read(address)
//----------------------------------------------------------- 
//EEPROM addresses
//const byte Ef_walking =0;
const byte Erollmin =1;
const byte ErollC   =2;
const byte Erollmax =3;
const byte Epacemin =4;
const byte EpaceC   =5;
const byte Epacemax =6;
const byte Erollby  =7;
const byte Epaceby  =8;

//-----------------------------------------------------------                                
Servo servoR;  // create servo object to control Roll servo 
Servo servoP;  // create servo object to control Pace servo 
//===========================================================
//edit this to your setup
//=======================
const byte servoRpin =7;      //Roll servo on pin 7   
const byte servoPpin =8;      //Pace servo on pin 8   
//===========================================================
byte Sframe =20; // Servo frame rate, same as Servo.h frame rate, no point in it being less   *new
//servos mid=90deg, range adjusted for a nominal range max of 100
byte roll ;    // servo position                                                              *new
byte pace ;    // servo position                                                              *new
byte rollby ;  // roll angles about rollC                                                     *new
byte paceby ;  // pace angles about paceC                                                     *new
byte cmnd =0;  // current command                                                             *new                          
boolean f_paceservo =true;       // for switch case                                           *new
byte rollC =EEPROM.read(ErollC);       // initialise rollC from current EEPROM ErollC         *new
byte paceC =EEPROM.read(EpaceC);       // initialise paceC from current EEPROM EpaceC         *new
//===========================================================
void setup() 
{ 
  if (rollC==255) {rollC =90;}      // in case it is not initialised                      *new
  if (paceC==255) {paceC =90;}      // in case it is not initialised                      *new
  EEPROM.write(EpaceC,paceC);
  EEPROM.write(ErollC,rollC);
 
  roll =rollC;
  pace =paceC;
  servoR.write(roll);            // tell servo to go to position rollC                    *new
  servoP.write(pace);            // tell servo to go to position paceC                    *new
  servoR.attach(servoRpin);      // attaches servoRpin to the servoR object 
  servoP.attach(servoPpin);      // attaches servoPpin to the servoP object 
  Serial.begin(9600);            // to communicate with Terminal Window   
  printInstructions();
  printEEPROMvalues();  
  Serial.println("First the Pace servo");
  Serial.print("pace =");
  Serial.println(pace);  
} 
//-----------------------------------------------------------   
void loop() 
{     
  if (Serial.available()>0) 
  {
    cmnd =Serial.read();
    switch (cmnd) 
    {
      case 'i':
        printInstructions();
        break;        
      case 'I':
        printInstructions();
        break;        
      case '*':
        printEEPROMvalues();
        break;        
      case '/':
        f_paceservo =!f_paceservo;     // !true ->false, !false ->true
        break;        
      case '-': 
        if (f_paceservo) 
          { pace -=1; }
         else 
          { roll -=1; }
        break;
      case '+': 
        if (f_paceservo) 
          { pace +=1; }
         else 
          { roll +=1; }
        break;
      case 'h': 
        if (f_paceservo) 
          {EEPROM.write(Epacemax,pace); Serial.print("pacemax ="); Serial.println(pace); }
         else 
          {EEPROM.write(Erollmax,roll); Serial.print("rollmax ="); Serial.println(roll); }
        break;
      case 'l': 
        if (f_paceservo) 
          {EEPROM.write(Epacemin,pace); Serial.print("pacemin ="); Serial.println(pace); }
         else 
          {EEPROM.write(Erollmin,roll); Serial.print("rollmin ="); Serial.println(roll); }
        break;
      case 'c': 
        if (f_paceservo) 
          {EEPROM.write(EpaceC,pace); Serial.print("paceCentre ="); Serial.println(pace); }
         else 
          {EEPROM.write(ErollC,roll); Serial.print("rollCentre ="); Serial.println(roll); }
        break;
      case 'r': 
        rollby =abs(roll -rollC); 
        EEPROM.write(Erollby,rollby); Serial.print("RollBy ="); Serial.println(rollby); 
        break;
      case 'p':
        paceby =abs(pace -paceC);  
        EEPROM.write(Epaceby,paceby); Serial.print("PaceBy ="); Serial.println(paceby);
        break;
      case 'H': 
        if (f_paceservo) 
          {EEPROM.write(Epacemax,pace); Serial.print("pacemax ="); Serial.println(pace); }
         else 
          {EEPROM.write(Erollmax,roll); Serial.print("rollmax ="); Serial.println(roll); }
        break;
      case 'L': 
        if (f_paceservo) 
          {EEPROM.write(Epacemin,pace); Serial.print("pacemin ="); Serial.println(pace); }
         else 
          {EEPROM.write(Erollmin,roll); Serial.print("rollmin ="); Serial.println(roll); }
        break;
      case 'C': 
        if (f_paceservo) 
          { paceC =pace;
            EEPROM.write(EpaceC,pace); Serial.print("paceCentre ="); Serial.println(pace); }
         else 
          { rollC =roll;
            EEPROM.write(ErollC,roll); Serial.print("rollCentre ="); Serial.println(roll); }
        break;
      case 'R':
        rollby =abs(roll -rollC); 
        EEPROM.write(Erollby,rollby); Serial.print("RollBy ="); Serial.println(rollby); 
        break;
      case 'P': 
        paceby =abs(pace -paceC); 
        EEPROM.write(Epaceby,paceby); Serial.print("PaceBy ="); Serial.println(paceby);
        break;
    }    // end switch
      
    
    if (f_paceservo) 
      { Serial.print("pace ="); Serial.println(pace); }
     else 
      { Serial.print("roll ="); Serial.println(roll); }

    servoR.write(roll);              // tell servo to go to position rollC
    servoP.write(pace);              // tell servo to go to position paceC
  }
} 
//===========================================================
void printEEPROMvalues()  {
  Serial.println("Current values in EEPROM");
  Serial.print("  Rollmin    =");    Serial.println(EEPROM.read(Erollmin));
  Serial.print("  RollCentre =");    Serial.println(EEPROM.read(ErollC));
  Serial.print("  Rollmax    =");    Serial.println(EEPROM.read(Erollmax));
  Serial.print("  Pacemin    =");    Serial.println(EEPROM.read(Epacemin));
  Serial.print("  PaceCentre =");    Serial.println(EEPROM.read(EpaceC));
  Serial.print("  Pacemax    =");    Serial.println(EEPROM.read(Epacemax));
  Serial.print("  PaceBy     =");    Serial.println(EEPROM.read(Epaceby));
  Serial.print("  RollBy     =");    Serial.println(EEPROM.read(Erollby));  
}

void printInstructions()  {
  Serial.println("Determine and save the maximum and minimum safe values");
  Serial.println("Determine and save the Servo centre position");
  Serial.println("Determine and save the default Pace and Roll values for walking");
  Serial.println("hit [+] or [-] to adjust the value and move the Servo");
  Serial.println("hit [H] to set the maximum safe value");
  Serial.println("hit [L] to set the minimum safe value");
  Serial.println("hit [C] to set the centre position value");
  Serial.println("hit [R] to set the default Pace length");
  Serial.println("hit [P] to set the default Roll amount");
  Serial.println("hit [/] to toggle between servos");  
  Serial.println("hit [I] to display instructions");
  Serial.println("hit [*] to display current EEPROM values");
}
 
