// A07_USmove
// added Ultrasonic routines for HC-SR04, readcm_HC_SR04(), Ul(x), Ue(x), Ug(x)
// added debug flag for debug messages to Serial Monitor or Terminal @9600 baud
// added ttyx(y) to print messages instead of having to type Serial.print(y)
// added Trollby and Twalkby for different roll and pace for turns

// Binary sketch size:  7,476 bytes (of a 14,336 byte maximum) / (of a 30,720 byte maximum)

/* 
 moves end with both feet flat on the ground
 moves start at paceat and end at paceto, paceat updated to paceto
 for walking and turning paceto == paceC +- paceby
         [x] are proposed character commands - not yet implemented

  Ht() - [H] Halt with both feet together
  w(wait) - [w(wait)] Wait for 1/10 seconds, w(10) wait 1 second    -*new, was D()
  Fd() - [F] Forward one pace
  Bk() - [B] Backward one pace
  Rt() - [R] Right turn one pace
  Lt() - [L] Left turn one pace
  In turns it may be necessary to step one pace so the feet 
    are in the right relation to each other for the turn,
    when going forward the steps are forward,
    when going backward the steps are backward.

  sp(sSpeed)      - [sp(sSpeed)] sets servo Speed, 0 - 255, default 2 
                                 sets both rollspeed and pacespeed to sSpeed
  Spmax(speedmax) - [sm(speedmax)] sets speedmax, maximum speed for su sd, 0 - 255, default advised 15
  sc(sby)         - [sc(sby)] sets speed change for su sd, default 1
  su()            - [su] speed Up, increment sSpeed every call
  sd()            - [sd] speed Down, decrement sSpeed every call
  s0()            - [s0] detach servos from servo object - stops pulsing the servos
  s1()            - [s1] re-attach servos to servo object - starts pulsing the servos

  Hr() - Right Foot step forward to centre
  Hl() - Left Foot step forward to centre
  Fr() - Forward with Right foot one pace
  Fl() - Forward with Left foot one pace
  Br() - Backward with Right foot one pace
  Bl() - Backward with Left  foot one pace

  rC() - roll to Roll Centre, ie upright
  rR() - roll to Right
  rL() - roll to Left
  
  Sliding does not lift the feet
  pC() - slide to Pace Centre, ie feet side by side
  Lf() - slide left foot Forward  w.r.t. right
  Rb() - slide left foot Forward  w.r.t. right
  Lb() - slide left foot Backward w.r.t. right
  Rf() - slide left foot Backward w.r.t. right
                                                               //*new
  dist_HC_SR04() - read HC-SR04 ultrasonic sensor, get distance
                   if f_cm ==true then dist in cm, else dist in inches
  Ul(x) - [Ul] Ultrasonic distance Less than (x), TRUE if ( x < dist_HC_SR04()) )
  Ue(x) - [Ue] Ultrasonic distance Equal to (x),  TRUE if ( x == dist_HC_SR04() )
  Ug(x) - [Ug] Ultrasonic distance Greater than (x), TRUE if ( x > dist_HC_SR04() )

  ttyl()             - Serial.println()  send NewLine to Serial Monitor or terminal
  ttyc("characters") - Serial.print("characters")  print characters
  ttyn(number)       - Serial.print(number)  print number

  Er(x) - EyeRight 1=on, 0=off
  El(x) - EyeRight 1=on, 0=off
  Vl(x) - VoiceLed 1=on, 0=off
  t(x)  - tone on D3, x in units of 100Hz, so t(3) is 300Hz 
  bh(x) - beep high tone for x in 1/10 seconds
  bm(x) - beep middle tone for x in 1/10 seconds
  bh(x) - beep low tone for x in 1/10 seconds

  debug =0 - turn off debug messages
  debug =1 - turn on debug messages
*/  
//===========================================================                               
// edit to select defaults for robot type

/*
// Bambino
  // HobbyKing HK15128 servo
  // +ve roll right, +ve left foot forward
  const byte Drollmin  = 60;
  const byte DrollC    = 90;
  const byte Drollmax  =120;
  const byte Dpacemin = 75;
  const byte DpaceC    = 90;
  const byte Dpacemax  =105;
  const byte Drollby   = 15;   // Default roll amount about rollC for walking
  const byte Dpaceby   = 15;   // Default pace amount about paceC for walking
  const byte DsSpeed   =  2;   // Default sSpeed for servos
  const byte Dsby      =  1;   // Default servo sSpeed increment for su sd
  const byte Dspeedmax =  4;   // Default maximum speed for servos
  const byte sframe    = 20;   // Servo frame rate, same as Servo.h frame rate, no point in it being less
  //  - For the Ardubino shield
  // --------------------------
    const byte Rx2       =2;     // serialIn from Bambino FaceShield
    const byte VoiceLed  =3;     // serialout / VoiceLed / IRin
    const byte Tx2       =4;     // serialOut to Bambino FaceShield
    const byte EyeLedR   =5;     // HMI 
    const byte EyeLedL   =6;     // HMI 
    const byte servoRpin =7;     // Roll servo on pin 7   
    const byte servoPpin =8;     // Pace servo on pin 8  
    const byte UStrigpin =9;     // Ultrasonic module Trigger pin, 
    const byte USechopin =9;     // Ultrasonic module Echo pin
// end Bambino 
*/

// Jet
  // Corona CS-538BP servo
  // +ve roll left, +ve right foot forward
  const byte Drollmin  = 60;
  const byte DrollC    = 90;
  const byte Drollmax  =120;
  const byte Dpacemin = 75;
  const byte DpaceC    = 90;
  const byte Dpacemax  =105;
  const byte Drollby   = 25;   // Default roll amount about rollC for walking
  const byte Dpaceby   = 40;   // Default pace amount about paceC for walking
  const byte DsSpeed   =  2;   // Default sSpeed for servos
  const byte Dsby      =  1;   // Default servo sSpeed increment for su sd
  const byte Dspeedmax =  4;   // Default maximum speed for servos
  const byte sframe    = 20;   // Servo frame rate, same as Servo.h frame rate, no point in it being less
  //  - For the V5 shield
  // --------------------
    const byte servoRpin =7;     // Roll servo on pin 7   
    const byte servoPpin =8;     // Pace servo on pin 8  
    const byte UStrigpin =A0;    // Ultrasonic module Trigger pin, 
    const byte USechopin =A1;    // Ultrasonic module Echo pin
  // not part of the V5 shield but could be added and are referenced in the code
  //----------------------------------------------------------------------------
    const byte Rx2       =2;     // serialIn from Bambino FaceShield
    const byte VoiceLed  =3;     // serialout / VoiceLed / IRin
    const byte Tx2       =4;     // serialOut to Bambino FaceShield
    const byte EyeLedR   =5;     // HMI 
    const byte EyeLedL   =6;     // HMI 
// end Jet
  
//===========================================================                               
#include <Servo.h>
Servo servoR;    // create servo object to control Roll servo 
Servo servoP;    // create servo object to control Pace servo 
/* Servo servoname - create servo object to control servo servoname  
   attach(pin) - Attaches a servo to an i/o pin.
   attach(pin,min,max) - Attaches to a pin setting min and max values in microseconds, 
                         default min is 544, max is 2400  
   write()     - Sets the servo angle in degrees.  
                 (invalid angle that is valid as pulse in microseconds is treated as microseconds)
   writeMicroseconds() - Sets the servo pulse width in microseconds 
   read()      - Gets the last written servo pulse width as an angle between 0 and 180. 
   readMicroseconds()  - Gets the last written servo pulse width in microseconds. (was read_us() in first release)
   attached()  - Returns true if there is a servo attached. 
   detach()    - Stops an attached i/o pin from pulsing its servo. 
 */ 
//=========================================================== 
#include <EEPROM.h>     // to read and write internal EEPROM
                        // EEPROM.write(address,bytevalue)
                        // bytevalue =EEPROM.read(address)
//----------------------------------------------------------- 
//EEPROM addresses
const byte Ef_walking =0;  // EEPROM address for f_walking boolean flag  
const byte Erollmin   =1;
const byte ErollC     =2;
const byte Erollmax   =3;
const byte Epacemin   =4;
const byte EpaceC     =5;
const byte Epacemax   =6;
const byte Ewrollby   =7;
const byte Ewpaceby   =8;
const byte Etrollby   =9;				// *new
const byte Etpaceby   =10;				// *new

//=========================================================== 
boolean debug;            //debug =0 - turn off debug messages, debug =1 - turn them off
boolean f_walking =true;  // walk/stop
boolean f_Fd      =true;  // false if last direction was backwards, true if forward 
boolean f_cm      =true;  // US distance in cm
byte rollC   =EEPROM.read(ErollC);
byte paceC   =EEPROM.read(EpaceC);
byte rollmin =EEPROM.read(Erollmin);
byte rollmax =EEPROM.read(Erollmax);
byte pacemin =EEPROM.read(Epacemin);
byte pacemax =EEPROM.read(Epacemax);
byte Wrollby =EEPROM.read(Ewrollby); 
byte Wpaceby =EEPROM.read(Ewpaceby); 
byte Trollby =EEPROM.read(Etrollby);                                     // *new
byte Tpaceby =EEPROM.read(Etpaceby);                                     // *new  
byte rollat ;   // variable to store the roll servo position 
byte paceat ;   // variable to store the pace servo position 
byte rollto;   // desired roll-servo position 
byte paceto;   // desired pace-servo position 
byte rollby    =Wrollby;  // roll amount for walking 
byte paceby    =Wpaceby;  // pace amount for walking 
byte sSpeed    =DsSpeed;  // servo speed for both servos
byte rollspeed =DsSpeed;  // roll servo speed  
byte pacespeed =DsSpeed;  // pace servo speed
byte speedmax  =Dspeedmax;// max speed for auto-increment/decrement
byte sby       =Dsby;     // speed change for auto-increment/decrement
byte param;                   //general parameter
//byte cmnd ;    // current command
//=========================================================== 
// Ultrasonic module HC-SR04
// According to Parallax's datasheet for the PING))), 
  // See: http://www.parallax.com/dl/docs/prod/acc/28015-PING-v1.3.pdf
  // sound travels at 1130 feet per second, i.e. 73.746 microseconds per inch
  // (or the speed of sound is 340 m/s or 29 microseconds per centimeter).
  // This gives the distance travelled by the ping, outbound and return,
  // so we divide by 2 to get the distance of the obstacle
  // and then divide by 29 to get cm or divide by 73.746 to get inches,
  // i.e. divide by 2*29=58 to get cm, 
  // or divide by 2*73.746=147.492 =>147 to get inches.
int UStime;
const int UStimeout =20000;     //20ms
const byte uscm     =58;
const byte usinch   =147;
//===========================================================
void setup()
{ 
  debug =1;                 // turn on debug printing
//  debug =0;                 // turn off debug printing
  f_walking =EEPROM.read(Ef_walking);   // read old state 
  f_walking =!f_walking ;			    // !true ->false, !false ->true
  EEPROM.write(Ef_walking,f_walking);   // save new state
  pinMode(UStrigpin, OUTPUT);
  // in case EEPROM not initialised
    if (rollC==255)   {rollC =DrollC;     EEPROM.write(ErollC,DrollC);}
    if (paceC==255)   {paceC =DpaceC;     EEPROM.write(EpaceC,DpaceC);}
    if (rollmin==255) {rollmin =Drollmin; EEPROM.write(Erollmin,Drollmin);}
    if (rollmax==255) {rollmax =Drollmax; EEPROM.write(Erollmax,Drollmax);}
    if (pacemin==255) {pacemin =Dpacemin; EEPROM.write(Epacemin,Dpacemin);}
    if (pacemax==255) {pacemax =Dpacemax; EEPROM.write(Epacemax,Dpacemax);}
    if (Wrollby==255) {Wrollby =Drollby;  EEPROM.write(Ewrollby,Drollby);}
    if (Wpaceby==255) {Wpaceby =Dpaceby;  EEPROM.write(Ewpaceby,Dpaceby);}
    if (Trollby==255) {Trollby =Drollby;  EEPROM.write(Etrollby,Drollby);}
    if (Tpaceby==255) {Tpaceby =Dpaceby;  EEPROM.write(Etpaceby,Dpaceby);}
  
  rollat =rollC;                 // initialise rollat
  paceat =paceC;                 // initialise paceat    
  
  servoR.write(rollC);           // tell servo to go to position rollC 
  servoP.write(paceC);           // tell servo to go to position paceC
  if (f_walking)  
    { servoR.attach(servoRpin);  // attaches servoRpin to the servoR object 
      servoP.attach(servoPpin);  // attaches servoPpin to the servoP object 
    }  	
  Serial.begin(9600);
  Serial.print("walking=");  Serial.println(f_walking);
  if (f_cm)  {Serial.print("Ultrasonic distance in cm"); }
  if (!f_cm) {Serial.print("Ultrasonic distance in inches"); }
  pinMode(EyeLedR, OUTPUT);
  pinMode(EyeLedL, OUTPUT);
  pinMode(VoiceLed, OUTPUT);

} 
//-----------------------------------------------------------  
void loop()
{
  /*ER(1);
  //t(30);
  w(2);
  bm(1);
  EL(1);
  w(2);
  bl(2);
  ER(0);
  w(2);
  EL(0);
  w(2);
  Vl(1);
  w(3);
  Vl(0);
  w(2);
  bh(2);
  */
  
  //ttyc("testing");
  ER(paceat<paceC);
  EL(paceat>paceC);
  
  Fd();     // take the next step and take avoiding action
  if(Ul(10)) {Vl(1);ttyc("<10"); bh(1);Bk();bh(1);Bk();}
  if(Ul(15)) {Vl(1);ttyc("<15"); bm(1);Rt();}
  if(Ul(20)) {Vl(1);ttyc("<20"); bl(1);Rt();}
  ttyl();Vl(0);

  /*Fd();  // walk forward, take avoiding action if cm<20, , alternative code
  cm =readcm_HC_SR04();
  byte rangecase =map(cm,0,20,0,3);
  //byte range =map(readcm_HC_SR04(),0,20,0,3);    //alternative code
  Serial.print("cm="); Serial.print(cm); Serial.print(" =>"); Serial.println(rangecase);
  switch (rangecase)
  { case 0: Bk();        // and fall through
    case 1: Bk();        // and fall through
    case 2: Rt();        // and fall through
    case 3: Rt(); break;
  } */         
}
//===========================================================
boolean Ul(int dist)   { if (readcm_HC_SR04() <=dist) {return true;}; return false; }    
boolean Ue(int dist)   { if (readcm_HC_SR04() ==dist) {return true;}; return false; }
boolean Ug(int dist)   { if (readcm_HC_SR04() >=dist) {return true;}; return false; }    
//----------------------------------------------------------- 
int readcm_HC_SR04()
{ if (debug) {Serial.print('U');}
  pinMode(UStrigpin,OUTPUT);
  digitalWrite(UStrigpin,HIGH);
  delayMicroseconds(10);                  // 10 recommended minimum
  digitalWrite(UStrigpin,LOW);
  pinMode(UStrigpin,INPUT);
  UStime =pulseIn(USechopin,HIGH,UStimeout);
  if (UStime==0) {UStime =UStimeout;}    // timeout returns 0, so make it a big distance
  //Serial.print(UStime);
  if (f_cm) {return UStime/uscm;} else {return UStime/usinch;}
}
//----------------------------------------------------------- 
void ER(byte x) {if (x==0) {digitalWrite(EyeLedR,LOW);} else {digitalWrite(EyeLedR,HIGH);}}
void EL(byte x) {if (x==0) {digitalWrite(EyeLedL,LOW);} else {digitalWrite(EyeLedL,HIGH);}}
void Vl(byte x) {if (x==0) {digitalWrite(VoiceLed,LOW);} else {digitalWrite(VoiceLed,HIGH);}}  //VoiceLed 1=on, 0=off
void t(byte x)  {if (x==0) {noTone(VoiceLed);} else {tone(VoiceLed,x/100);}} //tone on D3, x in units of 100Hz, so t(3) is 300Hz 
void bh(byte x) {tone(VoiceLed,4000,x*100);}  //beep high tone for x in 1/10 seconds
void bm(byte x) {tone(VoiceLed,2000,x*100);}  //beep middle tone for x in 1/10 seconds
void bl(byte x) {tone(VoiceLed,500,x*100);}   //beep low tone for x in 1/10 seconds
//----------------------------------------------------------- 
void ttyl()               { Serial.println(); }
void ttyc(char cparam[])  { Serial.print(cparam); }
void ttyn(int nparam)     { Serial.print(nparam); }
//-----------------------------------------------------------     
void w(byte x)  {if (debug) {Serial.print('w');} delay(x *100); }// Wait 1/10 seconds 	
void Ht()           { if (paceat <paceC) {Hr();} else {Hl();} }  // Halt with both feet together
void Hr()           { rL(); pC(); rC(); }                        // Right Foot step forward to centre
void Hl()           { rR(); pC(); rC(); }                        // Left Foot step forward to centre
void Fd()           { if (paceat >paceC) {Fr();} else {Fl();} }  // step forward one pace
void Fr()           { rL(); Rf(); rC(); }                        // Right Foot step forward one pace
void Fl()           { rR(); Lf(); rC(); }                        // Left Foot step forward one pace
void Bk()           { if (paceat >paceC) {Bl();} else {Br();} }  // step backward one pace
void Br()           { rL(); Rb(); rC(); }                        // Right Foot step backward one pace
void Bl()           { rR(); Lb(); rC(); }                        // Left Foot step backward one pace
void Lt()  { if (f_Fd) { if (paceat >paceC) {Fr();}; Lf(); }     // Left turn  
             else      { if (paceat >paceC) {Bl();}; Lf(); } }    
void Rt()  { if (f_Fd) { if (paceat <paceC) {Fl();}; Rf(); }     // Right turn 
             else      { if (paceat <paceC) {Br();}; Rf(); } } 
    
//----------------------------------------------------------- 
void sp(int param)    { sSpeed =constrain(param,0,255); }     // set servo speed              // keep it in a byte
void Spmax(int param) { speedmax =constrain(param,0,255); }   // set maximum speed for su()   // keep it in a byte
void sc(int param)    { sby  =constrain(param,0,255); }       // set speed change             // keep it in a byte
void su()                                                     // increment speed <=speedmax
  { int temp =sSpeed +sby;  sSpeed =constrain(temp,1,speedmax);
    rollspeed =sSpeed;  pacespeed =sSpeed; }
void sd()                                                     // decrement speed >0
  { int temp =sSpeed -sby;  sSpeed =constrain(temp,1,speedmax);
    rollspeed =sSpeed;  pacespeed =sSpeed; }
void s0()  { servoR.detach();  servoP.detach(); }             // detach servos so stop pulsing them     
void s1()                                                     // re-attach servos
  { if (!servoR.attached()) {servoR.attach(servoRpin); }
    if (!servoP.attached()) {servoP.attach(servoPpin); } }
	
//----------------------------------------------------------- 
void Rf() { paceto =paceC -paceby; pacemove(); }   // slide Rightfoot Forward  w.r.t. left  
void Lb() { paceto =paceC -paceby; pacemove(); }   // slide Rightfoot Forward  w.r.t. left
void Lf() { paceto =paceC +paceby; pacemove(); }   // slide Leftfoot Forward w.r.t. right
void Rb() { paceto =paceC +paceby; pacemove(); }   // slide Leftfoot Forward w.r.t. right
void pC() { paceto =paceC; pacemove(); }           // Pace Centre, move feet so they are side by side
void rC() { rollto =rollC; rollmove(); }           // roll to Centre
void rR() { rollto =rollC +rollby; rollmove(); }   // roll to Right
void rL() { rollto =rollC -rollby; rollmove(); }   // roll to Left   
  
//----------------------------------------------------------- 
void rollmove()    // do roll move  
  { if (rollspeed==0) {rollspeed =1;}     // dont hang if 0
    if (rollto > rollat)
      { for (rollat =rollat; rollat <rollto; rollat +=rollspeed)  {servoR.write(rollat); delay(sframe); } }  
     else     
      { for (rollat =rollat; rollat >rollto; rollat -=rollspeed)  {servoR.write(rollat); delay(sframe); } }
    rollat =rollto; servoR.write(rollat); delay(sframe);              // make sure servo is at rollto
  }

void pacemove()    // do pace move
  { if (pacespeed==0) {pacespeed =1;}     // dont hang if 0 
    if (paceto > paceat)
      { for (paceat =paceat; paceat <paceto; paceat +=pacespeed)  {servoP.write(paceat); delay(sframe); } }  
     else     
      { for (paceat =paceat; paceat >paceto; paceat -=pacespeed)  {servoP.write(paceat); delay(sframe); } }
    paceat =paceto; servoP.write(paceat); delay(sframe);              // make sure servo is at paceto
  }
//===========================================================


